@@ -3,13 +3,17 @@ from decimal import Decimal
 import locale
 import os
 import requests
+import warnings
 from flask import Flask, current_app, g
 from flask.ext import sqlalchemy
 from flask.ext.wtf.csrf import CsrfProtect
+from flask.ext.oauthlib.client import OAuth
 import six
 from sqlalchemy.engine import Engine
 from sqlalchemy.ext.declarative import declarative_base
 from sqlalchemy.schema import MetaData
+from werkzeug.utils import import_string
+from .transformers import Transformer
 
 
 db = sqlalchemy.SQLAlchemy()
@@ -36,7 +40,7 @@ _patch_metadata()
 from .util import DB_STATS, AcceptRequest
 
 
-__version__ = u'0.9.6'
+__version__ = u'0.9.7'
 
 
 requests_session = requests.Session()
@@ -49,17 +53,32 @@ locale.setlocale(locale.LC_ALL, '')
 csrf = CsrfProtect()
 
 
+oauth = OAuth()
+
+
 # Ensure models are declared
 from . import models
 from .auth import models as auth_models
+from .auth import AuthMethod
 
 
 def create_app(config=None, **kwargs):
     app = Flask('evesrp', **kwargs)
     app.request_class = AcceptRequest
     app.config.from_object('evesrp.default_config')
-    if config is not None:
-        app.config.from_pyfile(config)
+    # Check in config is a dict, python config file, or importable object name,
+    # in that order. Finally, check the EVESRP_SETTINGS environment variable
+    # as a last resort.
+    if isinstance(config, dict):
+        app.config.update(config)
+    elif isinstance(config, six.string_types):
+        if config.endswith(('.txt', '.py', '.cfg')):
+            app.config.from_pyfile(config)
+        else:
+            app.config.from_object(config)
+    elif config is None and 'EVESRP_SETTINGS' in os.environ:
+        app.config.from_envvar('EVESRP_SETTINGS')
+    # Look for the secret key config value as an environment variable
     if app.config['SECRET_KEY'] is None and 'SECRET_KEY' in os.environ:
         app.config['SECRET_KEY'] = os.environ['SECRET_KEY']
 
@@ -77,6 +96,10 @@ def create_app(config=None, **kwargs):
     # is the template function.
     app.before_request_funcs[None] = before_csrf
 
+    # Hook up OAuth
+    oauth.init_app(app)
+
+    # Connect views
     from .views import index, error_page, divisions, login, requests, api
     app.add_url_rule(rule=u'/', view_func=index)
     for error_code in (400, 403, 404, 500):
@@ -93,11 +116,6 @@ def create_app(config=None, **kwargs):
     from .json import SRPEncoder
     app.json_encoder=SRPEncoder
 
-    app.before_first_request(_copy_config_to_authmethods)
-    app.before_first_request(_config_requests_session)
-    app.before_first_request(_config_killmails)
-    app.before_first_request(_copy_url_converter_config)
-
     # Configure the Jinja context
     # Inject variables into the context
     from .auth import PermissionType
@@ -114,9 +132,18 @@ def create_app(config=None, **kwargs):
     app.jinja_env.trim_blocks = True
     app.jinja_env.lstrip_blocks = True
 
+    init_app(app)
+
     return app
 
 
+def init_app(app):
+    _config_requests_session(app)
+    _config_url_converters(app)
+    _config_authmethods(app)
+    _config_killmails(app)
+
+
 # SQLAlchemy performance logging
 def sqlalchemy_before():
     if DB_STATS.total_queries > 0:
@@ -127,49 +154,99 @@ def sqlalchemy_before():
     g.DB_STATS = DB_STATS
 
 
+# Utility function for creating instances from dicts
+def _instance_from_dict(instance_descriptor):
+    type_name = instance_descriptor.pop('type')
+    Type = import_string(type_name)
+    return Type(**instance_descriptor)
+
+
+# Utility function for raising config deprecation warnings
+def _deprecated_object_instance(key, value):
+    warnings.warn(u"Non-basic data types in configuration values are deprecated"
+                 u"({}: {})".format(key, value), DeprecationWarning,
+                 stacklevel=2)
+
+
 # Auth setup
-def _copy_config_to_authmethods():
-    current_app.auth_methods = current_app.config['SRP_AUTH_METHODS']
+def _config_authmethods(app):
+    auth_methods = []
+    # Once the deprecated config value support is removed, this can be
+    # rewritten as a dict comprehension
+    for method in app.config['SRP_AUTH_METHODS']:
+        if isinstance(method, dict):
+            auth_methods.append(_instance_from_dict(method))
+        elif isinstance(method, AuthMethod):
+            _deprecated_object_instance('SRP_AUTH_METHODS', method)
+            auth_methods.append(method)
+    app.auth_methods = auth_methods
 
 
 # Request detail URL setup
-def _copy_url_converter_config():
+def _config_url_converters(app):
     url_transformers = {}
-    for config_key, config_value in current_app.config.items():
+    for config_key, config_value in app.config.items():
+        # Skip null config values
         if config_value is None:
             continue
-        index = config_key.rfind('_URL_TRANSFORMERS')
+        # Look for config keys in the format 'SRP_*_URL_TRANSFORMERS'
         if not config_key.endswith('_URL_TRANSFORMERS')\
                 or not config_key.startswith('SRP_'):
             continue
         attribute = config_key.replace('SRP_', '', 1)
         attribute = attribute.replace('_URL_TRANSFORMERS', '', 1)
         attribute = attribute.lower()
-        url_transformers[attribute] = {t.name:t for t in config_value}
-    current_app.url_transformers = url_transformers
-    # temporary compatibility
-    current_app.ship_urls = url_transformers.get('ship_type', None)
-    current_app.pilot_urls = url_transformers.get('pilot', None)
+        # Create Transformer instances for this attribute
+        transformers = {}
+        for transformer_config in config_value:
+            if isinstance(transformer_config, tuple):
+                # Special case for Transformers: A 2-tuple in the form:
+                # (u'Transformer Name',
+                #     'http://example.com/transformer/slug/{}')
+                transformer = Transformer(*transformer_config)
+            elif isinstance(transformer_config, dict):
+                # Standard instance dictionary format
+                # Provide a default type value
+                transformer_config.setdefault('type',
+                        'evesrp.transformers.Transformer')
+                transformer = _instance_from_dict(transformer_config)
+            elif isinstance(transformer_config, Transformer):
+                # DEPRECATED: raw Transformer instance
+                _deprecated_object_instance(config_key, transformer_config)
+                transformer = transformer_config
+            transformers[transformer.name] = transformer
+        url_transformers[attribute] = transformers
+    app.url_transformers = url_transformers
 
 
 # Requests session setup
-def _config_requests_session():
+def _config_requests_session(app):
     try:
-        ua_string = current_app.config['SRP_USER_AGENT_STRING']
+        ua_string = app.config['SRP_USER_AGENT_STRING']
     except KeyError as outer_exc:
         try:
             ua_string = 'EVE-SRP/{version} ({email})'.format(
-                    email=current_app.config['SRP_USER_AGENT_EMAIL'],
+                    email=app.config['SRP_USER_AGENT_EMAIL'],
                     version=__version__)
         except KeyError as inner_exc:
             raise inner_exc
+    requests_session = requests.Session()
     requests_session.headers.update({'User-Agent': ua_string})
-    current_app.user_agent = ua_string
+    app.requests_session = requests_session
 
 
 # Killmail verification
-def _config_killmails():
-    current_app.killmail_sources = current_app.config['SRP_KILLMAIL_SOURCES']
+def _config_killmails(app):
+    killmail_sources = []
+    # For now, use a loop with checks. After removing the depecated config
+    # method it can be rewritten as a list comprehension
+    for source in app.config['SRP_KILLMAIL_SOURCES']:
+        if isinstance(source, six.string_types):
+            killmail_sources.append(import_string(source))
+        elif isinstance(source, type):
+            _deprecated_object_instance('SRP_KILLMAIL_SOURCES', source)
+            killmail_sources.append(source)
+    app.killmail_sources = killmail_sources
 
 
 # Work around DBAPI-specific issues with Decimal subclasses.
